var moment = require("moment");
var HourMinute = (function () {
    function HourMinute(origin) {
        this._duration = moment.duration(origin.hour, 'hours').add(origin.minute, 'minutes');
        this._justify();
    }
    // MARK: - non-mutating manipulation methods
    HourMinute.prototype.adding = function (time) {
        var duration = moment.duration(this._duration);
        duration.add(time.hour, 'hours');
        duration.add(time.minute, 'minutes');
        return new HourMinute({
            hour: duration.hours(),
            minute: duration.minutes()
        });
    };
    HourMinute.prototype.subtracting = function (time) {
        var duration = moment.duration(this._duration);
        duration.subtract(time.minute, 'minutes');
        duration.subtract(time.hour, 'hours');
        return new HourMinute({
            hour: duration.hours(),
            minute: duration.minutes()
        });
    };
    // MARK: - mutating manipulation methods
    HourMinute.prototype.add = function (time) {
        this._duration.add(time.hour, 'hours').add(time.minute, 'minutes');
        this._justify();
        return this;
    };
    HourMinute.prototype.addHours = function (hours) {
        return this.add({ hour: hours, minute: 0 });
    };
    HourMinute.prototype.addMinutes = function (minutes) {
        return this.add({ hour: 0, minute: minutes });
    };
    HourMinute.prototype.subtract = function (time) {
        this._duration.subtract((new HourMinute(time))._duration);
        this._justify();
        return this;
    };
    HourMinute.prototype.subtractHours = function (hours) {
        return this.subtract({ hour: hours, minute: 0 });
    };
    HourMinute.prototype.subtractMinutes = function (minutes) {
        return this.subtract({ hour: 0, minute: minutes });
    };
    HourMinute.prototype.snapToMinutes = function (perMinutes) {
        var overhead = this.minute % perMinutes;
        this._duration.subtract(overhead, 'minutes');
        this._justify();
        return this;
    };
    // MARK: - exportation methods
    HourMinute.prototype.asHours = function () {
        return this._duration.asHours();
    };
    HourMinute.prototype.asMinutes = function () {
        return this._duration.asMinutes();
    };
    HourMinute.prototype.format = function () {
        return this.hour + ':' + this.minute;
    };
    // MARK: - private methods
    HourMinute.prototype._justify = function () {
        this.hour = this._duration.hours();
        this.minute = this._duration.minutes();
    };
    return HourMinute;
})();
exports.HourMinute = HourMinute;
